@@ -1,8 +1,12 @@
 """Library for traits about devices."""
 
 import datetime
+from typing import Any, Dict, Mapping, Optional
+
+import aiohttp
 
 from .traits import TRAIT_MAP, Command
+from .typing import cast_assert, cast_optional
 
 STATUS = "status"
 TIMER_MODE = "timerMode"
@@ -18,7 +22,7 @@ class ConnectivityTrait:
 
     NAME = "sdm.devices.traits.Connectivity"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize ConnectivityTrait."""
         self._data = data
 
@@ -29,7 +33,7 @@ class ConnectivityTrait:
         Return:
           "OFFLINE", "ONLINE"
         """
-        return self._data[STATUS]
+        return cast_assert(str, self._data[STATUS])
 
 
 @TRAIT_MAP.register()
@@ -38,31 +42,33 @@ class FanTrait:
 
     NAME = "sdm.devices.traits.Fan"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize FanTrait."""
         self._data = data
         self._cmd = cmd
 
     @property
-    def timer_mode(self) -> str:
+    def timer_mode(self) -> Optional[str]:
         """Timer mode for the fan.
 
         Return:
           "ON", "OFF"
         """
-        return self._data.get(TIMER_MODE)
+        return cast_optional(str, self._data.get(TIMER_MODE))
 
     @property
-    def timer_timeout(self) -> datetime.datetime:
+    def timer_timeout(self) -> Optional[datetime.datetime]:
         """Timestamp at which timer mode will turn to OFF."""
         if TIMER_TIMEOUT not in self._data:
             return None
         timestamp = self._data[TIMER_TIMEOUT]
         return datetime.datetime.fromisoformat(timestamp.replace("Z", "+00:00"))
 
-    async def set_timer(self, timer_mode, duration=None):
+    async def set_timer(
+        self, timer_mode: str, duration: Optional[int] = None
+    ) -> aiohttp.ClientResponse:
         """Change the fan timer."""
-        data = {
+        data: Dict[str, Any] = {
             "command": "sdm.devices.commands.Fan.SetTimer",
             "params": {
                 "timerMode": timer_mode,
@@ -79,14 +85,14 @@ class InfoTrait:
 
     NAME = "sdm.devices.traits.Info"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize InfoTrait."""
         self._data = data
 
     @property
-    def custom_name(self) -> str:
+    def custom_name(self) -> Optional[str]:
         """Name of the device."""
-        return self._data[CUSTOM_NAME]
+        return cast_optional(str, self._data.get(CUSTOM_NAME))
 
 
 @TRAIT_MAP.register()
@@ -95,14 +101,14 @@ class HumidityTrait:
 
     NAME = "sdm.devices.traits.Humidity"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize HumidityTrait."""
         self._data = data
 
     @property
     def ambient_humidity_percent(self) -> float:
         """Percent humidity, measured at the device."""
-        return self._data[AMBIENT_HUMIDITY_PERCENT]
+        return cast_assert(float, self._data[AMBIENT_HUMIDITY_PERCENT])
 
 
 @TRAIT_MAP.register()
@@ -111,11 +117,11 @@ class TemperatureTrait:
 
     NAME = "sdm.devices.traits.Temperature"
 
-    def __init__(self, data: dict, cmd: Command):
+    def __init__(self, data: Mapping[str, Any], cmd: Command):
         """Initialize TemperatureTrait."""
         self._data = data
 
     @property
     def ambient_temperature_celsius(self) -> float:
         """Percent humidity, measured at the device."""
-        return self._data[AMBIENT_TEMPERATURE_CELSIUS]
+        return cast_assert(float, self._data[AMBIENT_TEMPERATURE_CELSIUS])
