@@ -1,13 +1,16 @@
 """Events from pubsub subscriber."""
 
+from __future__ import annotations
+
 import datetime
 import logging
 from abc import ABC
-from typing import Awaitable, Callable
+from typing import Any, Awaitable, Callable, Dict, Mapping, Optional
 
 from .auth import AbstractAuth
 from .registry import Registry
 from .traits import BuildTraits, Command
+from .typing import cast_assert, cast_optional
 
 EVENT_ID = "eventId"
 EVENT_SESSION_ID = "eventSessionId"
@@ -36,7 +39,7 @@ class EventProcessingError(Exception):
 class ImageEventBase(ABC):
     """Base class for all image related event types."""
 
-    def __init__(self, data: dict, timestamp: datetime.datetime):
+    def __init__(self, data: Mapping[str, Any], timestamp: datetime.datetime) -> None:
         """Initialize EventBase."""
         self._data = data
         self._timestamp = timestamp
@@ -47,12 +50,12 @@ class ImageEventBase(ABC):
 
         Can be used with CameraEventImageTrait to download the imaage.
         """
-        return self._data[EVENT_ID]
+        return cast_assert(str, self._data[EVENT_ID])
 
     @property
     def event_session_id(self) -> str:
         """ID used to associate separate messages with a single event."""
-        return self._data[EVENT_SESSION_ID]
+        return cast_assert(str, self._data[EVENT_SESSION_ID])
 
     @property
     def timestamp(self) -> datetime.datetime:
@@ -96,17 +99,17 @@ class DoorbellChimeEvent(ImageEventBase):
 class EventTrait(ABC):
     """Parent class for traits related to handling events."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize an EventTrait."""
-        self._last_event = None
+        self._last_event: Optional[ImageEventBase] = None
 
     @property
-    def last_event(self) -> ImageEventBase:
+    def last_event(self) -> Optional[ImageEventBase]:
         """Last received event."""
         return self._last_event
 
     @property
-    def active_event(self) -> ImageEventBase:
+    def active_event(self) -> Optional[ImageEventBase]:
         """Any current active events."""
         if not self._last_event:
             return None
@@ -116,7 +119,7 @@ class EventTrait(ABC):
             return None
         return self._last_event
 
-    def handle_event(self, event: ImageEventBase):
+    def handle_event(self, event: ImageEventBase) -> None:
         """Recieve an event message."""
         self._last_event = event
 
@@ -124,50 +127,54 @@ class EventTrait(ABC):
 class RelationUpdate:
     """Represents a relational update for a resource."""
 
-    def __init__(self, raw_data: dict):
+    def __init__(self, raw_data: Mapping[str, Any]):
         """Initialize the RelationUpdate."""
         self._raw_data = raw_data
 
     @property
     def type(self) -> str:
         """Type of relation event 'CREATED', 'UPDATED', 'DELETED'."""
-        return self._raw_data[TYPE]
+        return cast_assert(str, self._raw_data[TYPE])
 
     @property
     def subject(self) -> str:
         """Resource that the object is now in relation with."""
-        return self._raw_data[SUBJECT]
+        return cast_assert(str, self._raw_data.get(SUBJECT))
 
     @property
     def object(self) -> str:
         """Resource that triggered the event."""
-        return self._raw_data[OBJECT]
+        return cast_assert(str, self._raw_data[OBJECT])
 
 
-def BuildEvents(events: dict, event_map: dict, timestamp: datetime.datetime) -> dict:
+def BuildEvents(
+    events: Mapping[str, Any],
+    event_map: Mapping[str, ImageEventBase],
+    timestamp: datetime.datetime,
+) -> Dict[str, ImageEventBase]:
     """Build a trait map out of a response dict."""
     result = {}
     for (event, event_data) in events.items():
         if event not in event_map:
             continue
         cls = event_map[event]
-        result[event] = cls(event_data, timestamp)
+        result[event] = cls(event_data, timestamp)  # type: ignore
     return result
 
 
 class EventMessage:
     """Event for a change in trait value or device action."""
 
-    def __init__(self, raw_data: dict, auth: AbstractAuth):
+    def __init__(self, raw_data: Mapping[str, Any], auth: AbstractAuth):
         """Initialize an EventMessage."""
         _LOGGER.debug("EventMessage raw_data=%s", raw_data)
         self._raw_data = raw_data
         self._auth = auth
 
     @property
-    def event_id(self) -> str:
+    def event_id(self) -> Optional[str]:
         """Event identifier."""
-        return self._raw_data.get(EVENT_ID, None)
+        return self._raw_data.get(EVENT_ID)
 
     @property
     def timestamp(self) -> datetime.datetime:
@@ -176,54 +183,57 @@ class EventMessage:
         return datetime.datetime.fromisoformat(event_timestamp.replace("Z", "+00:00"))
 
     @property
-    def resource_update_name(self) -> str:
+    def resource_update_name(self) -> Optional[str]:
         """Return the id of the device that was updated."""
         if RESOURCE_UPDATE not in self._raw_data:
             return None
-        return self._raw_data[RESOURCE_UPDATE][NAME]
+        return cast_optional(str, self._raw_data[RESOURCE_UPDATE].get(NAME))
 
     @property
-    def resource_update_events(self) -> dict:
+    def resource_update_events(self) -> Optional[Dict[str, ImageEventBase]]:
         """Return the set of events that happened."""
         if RESOURCE_UPDATE not in self._raw_data:
             return None
         events = self._raw_data[RESOURCE_UPDATE].get(EVENTS, {})
+        assert isinstance(events, dict)
         return BuildEvents(events, EVENT_MAP, self.timestamp)
 
     @property
-    def resource_update_traits(self) -> dict:
+    def resource_update_traits(self) -> Optional[dict]:
         """Return the set of traits that were updated."""
-        if RESOURCE_UPDATE not in self._raw_data:
+        if not self.resource_update_name:
             return None
         cmd = Command(self.resource_update_name, self._auth)
         events = self._raw_data[RESOURCE_UPDATE].get(TRAITS, {})
         return BuildTraits(events, cmd)
 
     @property
-    def relation_update(self) -> RelationUpdate:
+    def relation_update(self) -> Optional[RelationUpdate]:
         """Represent a relational update for a resource."""
         if RELATION_UPDATE not in self._raw_data:
             return None
         return RelationUpdate(self._raw_data[RELATION_UPDATE])
 
     @property
-    def raw_data(self) -> str:
-        """Return the raw data string."""
-        return self._raw_data
+    def raw_data(self) -> Dict[str, Any]:
+        """Return raw data for the event."""
+        return dict(self._raw_data)
 
 
 class EventTypeFilterCallback:
     """Invoke a delegate only for events that match the trait type."""
 
-    def __init__(self, event_name, delegate: Callable[[EventMessage], Awaitable[None]]):
+    def __init__(
+        self, event_name: str, delegate: Callable[[EventMessage], Awaitable[None]]
+    ) -> None:
         """Initialize EventTypeFilterCallback."""
         self._event_name = event_name
         self._delegate = delegate
 
-    async def async_handle_event(self, event_message: EventMessage):
+    async def async_handle_event(self, event_message: EventMessage) -> None:
         """Process an incoming EventMessage."""
         events = event_message.resource_update_events
-        if self._event_name in events:
+        if events and self._event_name in events:
             await self._delegate(event_message)
 
 
@@ -239,7 +249,7 @@ class RecentEventFilterCallback:
         self._cutoff_timedelta = cutoff_timedelta
         self._delegate = delegate
 
-    async def async_handle_event(self, event_message: EventMessage):
+    async def async_handle_event(self, event_message: EventMessage) -> None:
         """Process an incoming EventMessage."""
         if not event_message.timestamp:
             return
