@@ -22,6 +22,7 @@ import json
 import logging
 import os
 import pickle
+from typing import Optional, List, cast
 
 import yaml
 from aiohttp import ClientSession
@@ -30,6 +31,8 @@ from google.auth.transport.requests import Request
 from google_auth_oauthlib.flow import InstalledAppFlow
 
 from .auth import AbstractAuth
+from .structure import Structure
+from .device import Device
 from .camera_traits import CameraLiveStreamTrait
 from .event import EventMessage
 from .google_nest_api import GoogleNestAPI
@@ -133,16 +136,16 @@ class Auth(AbstractAuth):
         super().__init__(websession, api_url)
         self._user_creds = user_creds
 
-    async def async_get_access_token(self):
+    async def async_get_access_token(self) -> str:
         """Return a valid access token."""
-        return self._user_creds.token
+        return cast(str, self._user_creds.token)
 
-    async def async_get_creds(self):
+    async def async_get_creds(self) -> Credentials:
         """Return valid OAuth creds."""
         return self._user_creds
 
 
-def CreateCreds(args) -> Credentials:
+def CreateCreds(args: argparse.Namespace) -> Credentials:
     """Run an interactive flow to get OAuth creds."""
     creds = None
     token_cache = os.path.expanduser(args.token_cache)
@@ -182,7 +185,7 @@ def CreateCreds(args) -> Credentials:
     return creds
 
 
-def PrintStructure(structure, output_type):
+def PrintStructure(structure: Structure, output_type: str) -> None:
     """Print the structure."""
     if output_type == "json":
         print(json.dumps(structure.raw_data))
@@ -190,7 +193,7 @@ def PrintStructure(structure, output_type):
         print(yaml.dump(structure.raw_data))
 
 
-def PrintDevice(device, output_type):
+def PrintDevice(device: Device, output_type: str) -> None:
     """Print the device."""
     if output_type == "json":
         print(json.dumps(device.raw_data))
@@ -201,11 +204,11 @@ def PrintDevice(device, output_type):
 class SubscribeCallback:
     """Print the event message."""
 
-    def __init__(self, output_type=None):
+    def __init__(self, output_type: Optional[str] = None) -> None:
         """Initialize SubscribeCallback."""
         self._output_type = output_type
 
-    async def async_handle_event(self, event_message: EventMessage):
+    async def async_handle_event(self, event_message: EventMessage) -> None:
         """Handle an EventMessage."""
         if self._output_type == "json":
             print(json.dumps(event_message.raw_data))
@@ -216,12 +219,12 @@ class SubscribeCallback:
 class DeviceWatcherCallback:
     """Print the event message."""
 
-    def __init__(self, device, output_type):
+    def __init__(self, device: Device, output_type: str) -> None:
         """Initialize DeviceWatcherCallback."""
         self._device = device
         self._output_type = output_type
 
-    async def async_handle_event(self, event_message: EventMessage):
+    async def async_handle_event(self, event_message: EventMessage) -> None:
         """Handle an EventMessage."""
         print(f"event_id: {event_message.event_id}")
         print("Current device state:")
@@ -229,27 +232,30 @@ class DeviceWatcherCallback:
         print("")
 
 
-async def RunTool(args, user_creds: Credentials):
+async def RunTool(args: argparse.Namespace, user_creds: Credentials) -> None:
     """Run the command."""
     async with ClientSession() as client:
         auth = Auth(client, user_creds, API_URL)
         api = GoogleNestAPI(auth, args.project_id)
 
         if args.command == "list_structures":
-            structures = await api.async_get_structures()
-            for structure in structures:
-                PrintStructure(structure, args.output_type)
+            structures: List[Structure] = await api.async_get_structures()
+            for s in structures:
+                PrintStructure(s, args.output_type)
             return
 
         if args.command == "get_structure":
-            structure = await api.async_get_structure(args.structure_id)
+            structure: Optional[Structure] = await api.async_get_structure(
+                args.structure_id
+            )
+            assert structure
             PrintStructure(structure, args.output_type)
             return
 
         if args.command == "list_devices":
             devices = await api.async_get_devices()
-            for device in devices:
-                PrintDevice(device, args.output_type)
+            for d in devices:
+                PrintDevice(d, args.output_type)
             return
 
         if args.command == "subscribe":
@@ -259,12 +265,12 @@ async def RunTool(args, user_creds: Credentials):
             )
             if args.device_id:
                 device_manager = await subscriber.async_get_device_manager()
-                device = device_manager.devices[args.device_id]
-                callback = DeviceWatcherCallback(device, args.output_type)
-                device.add_event_callback(callback.async_handle_event)
+                dev = device_manager.devices[args.device_id]
+                dev_callback = DeviceWatcherCallback(dev, args.output_type)
+                dev.add_event_callback(dev_callback.async_handle_event)
             else:
-                callback = SubscribeCallback(args.output_type)
-                subscriber.set_update_callback(callback.async_handle_event)
+                sub_callback = SubscribeCallback(args.output_type)
+                subscriber.set_update_callback(sub_callback.async_handle_event)
             await subscriber.start_async()
             try:
                 while True:
@@ -273,7 +279,8 @@ async def RunTool(args, user_creds: Credentials):
                 subscriber.stop_async()
 
         # All other commands require a device_id
-        device = await api.async_get_device(args.device_id)
+        device: Optional[Device] = await api.async_get_device(args.device_id)
+        assert device
 
         if args.command == "get_device":
             PrintDevice(device, args.output_type)
@@ -309,9 +316,9 @@ async def RunTool(args, user_creds: Credentials):
             print(f"Expires At: {stream.expires_at}")
 
 
-def main():
+def main() -> None:
     """Nest command line tool."""
-    args = parser.parse_args()
+    args: argparse.Namespace = parser.parse_args()
     if args.verbose:
         logging.basicConfig(level=logging.DEBUG)
     user_creds = CreateCreds(args)
